import java.io.*;
import java.awt.*;	
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.imageio.*;


public class JBoardPanel 
	extends JPanel 
	implements 	MouseListener, MouseMotionListener, 
				MouseWheelListener, ActionListener {

	private JWB_Tafel Parent = null;

	private static Color cBlackBoard 	= new Color(30, 60, 30);
	private static Color cLines 		= new Color(127, 192, 255);
	private static Color cMoverBack 	= new Color(200, 200, 200);
	private static Color cInfoBack 		= new Color(230, 230, 230);

	private static final int EMPTY = 0;
	private static final int LINES = 1;
	private static final int KAROS = 2;
	private static final int NOTES = 3;

	private static final int MODE_PENCIL = 0;
	private static final int MODE_LINEAL = 1;
	private static final int MODE_ERASER = 2;
	private static final int MODE_MOVER  = 3;
	private static final int MODE_NOTES  = 4;
	private static final int MODE_MUSIC  = 5;
	private static final int MODE_GEODR  = 6;

	private static int dLines = 50;
	private static int dKaros = 40;

	private static int dNotesA = 100;
	private static int dNotesB = 20;

	private static JEraseCursor[] erasers = new JEraseCursor[3];
	private int EraseStyle = 1;
	
	private Cursor defCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
	private Cursor penCursor = null;
	private Cursor linCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
	private Cursor delCursor = null;
	private Cursor movCursor = Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR);
	private Cursor notCursor = defCursor;
	
	private int backStyle = EMPTY;

	public JBoardImage image;
	public JBackground background;
	
	private Point startPoint = null;
	private Point endPoint = null;
	
	private boolean dragged;

	private int boardMode = MODE_PENCIL;

	private int xMin, xMax, yMin, yMax, erBreite, erHoehe;

	private static final int maxX = 2000;
	public static final int maxY = 8000;
	private static final int wheelY = 60;
	
	public int offset = 0;

	private boolean mouseaction = false;

	public JInfolabel infolabel;

	//--- Notenbilder

	private JNotePics NotePics;
	private int eineNote, eineMusic;
	
	//--- Mover-Vorschau
	
	private int moveOffset = 20;
	private int moveWidth  = 60;
	private int moveHeight;

	//--- GeoDreieck
	
	private JGeodreieck Geodreieck;

	//--- Popup-Menu
	
	private JPopup popup;
	
	//------------------------------------------------------------------

	public JBoardPanel (JWB_Tafel p) {

		setLayout(null);

		Parent = p;
		setBackground(Color.WHITE);

		image = new JBoardImage(maxX, maxY);
		background = new JBackground(maxX, maxY);
		background.setColor(cLines);

		addMouseListener(this);
		addMouseMotionListener(this);
		addMouseWheelListener(this);

		erasers[0] = new JEraseCursor(11,11);
		erasers[1] = new JEraseCursor(21,21);
		erasers[2] = new JEraseCursor(31,31);

		NotePics = new JNotePics();
		eineNote = 0;
		eineMusic = 0;

		infolabel = new JInfolabel(this);
		add(infolabel);

		Geodreieck = new JGeodreieck(this);
		add(Geodreieck);

		popup = new JPopup((ActionListener)this);
		setComponentPopupMenu(popup);

	}

	public void setBackStyle(int style) {
		backStyle = style;
		background.clearFrom(offset);

		switch(backStyle){
			case EMPTY : break;
			case LINES : background.paintLines(offset, dLines); break;
			case KAROS : background.paintKaros(offset, dKaros); break;
			case NOTES : background.paintNotes(offset, dNotesA, dNotesB); break;
		}

		repaint();
	}

	public void setPenColor(Color c) {
		image.setPenColor(c);
	}

	public void setPenCursor(Cursor c) {
		penCursor = c;
	}

	public void setPenStyle(int style) {
		image.setPenStyle(style);
		EraseStyle = style;
	}

	public void setBoardMode(int mode) {
		boardMode = mode;
		if (mode == MODE_MOVER) { moveHeight = (moveWidth*maxY)/getWidth(); }

		if (mode == MODE_GEODR) {
			Geodreieck.setVisible(true); repaint();
			} else {
			Geodreieck.setVisible(false);
			}

		repaint();
	}

	public void scroll(int dy) {
		if ( offset+dy < 0 ) { 
			offset = 0;
		} else if ( offset+dy+getHeight() <= maxY ) {
			offset = offset + dy;
		} else {
			/* int toMove = offset+dy+getHeight()-maxY;
			JBoardImage newimage = new JBoardImage(maxX, maxY);
			newimage.cloneSettingsFrom(image);
			newimage.drawImage( image.getSubimage(0, toMove, maxX, maxY-toMove),0,0);
			image = newimage; */
			offset = maxY-getHeight();
		}
		repaint();
	}

	public void scrollTo(int y) {
		offset = y;
		repaint();
	}


	public void clearWindow() {
		image.clear(0,offset, maxX, getHeight() ); repaint();
	}

	public void clearAll() {
		image.clear(0,0, maxX,maxY); repaint();
	}

	public String getLineAngle() {
		if (startPoint != null && endPoint != null)
		{
			double dx = endPoint.x-startPoint.x;
			double dy = startPoint.y-endPoint.y;

			double w = 0;

			if (dx == 0) {
				if ( dy > 0 ) { w = 90; }
				if ( dy < 0 ) { w = 270; }
			} else {
				w = Math.atan(dy/dx)*180/Math.PI;
				if (dx < 0) { w = w + 180; }
				if (dx > 0 && dy < 0 ) { w = w + 360; }
			}
			

			return String.format("%,5.1f°", w);
		} else return "";
	}

	public String getLineLength() {
		if (startPoint != null && endPoint != null)
		{
			double d = Math.sqrt( 
						(startPoint.x-endPoint.x)*(startPoint.x-endPoint.x)+
						(startPoint.y-endPoint.y)*(startPoint.y-endPoint.y)
						)/dKaros;
			return String.format("%,4.1f", d);
		} else return "";
	}

	//-------------------------------------------------------------------

	public void mouseEntered(MouseEvent e)
	{
		switch(boardMode) {
			case MODE_PENCIL : setCursor(penCursor); break;
			case MODE_LINEAL : setCursor(linCursor); break;
			case MODE_ERASER : setCursor(erasers[EraseStyle].getCursor()); 
							   break;
			case MODE_MOVER  : setCursor(movCursor); break;
			case MODE_NOTES  : setCursor(notCursor); break;
			case MODE_MUSIC  : setCursor(notCursor); break;
		}
	   repaint();

	}

	public void mouseExited(MouseEvent e)
	{
		setCursor(defCursor);
	}

	public void mouseClicked(MouseEvent e)	{	}
	public void mouseMoved(MouseEvent e)	{	}

	public void mousePressed(MouseEvent e)
	{
		if ( e.getButton() == 1 ) {

		mouseaction = true;
		dragged 	= false;
		startPoint 	= e.getPoint();
		endPoint 	= startPoint;

		switch(boardMode) {

			case MODE_PENCIL : 
				image.setColor();
				image.setStroke();
				break;

			case MODE_LINEAL : 
				image.setColor();
				image.setStroke();
				xMin = startPoint.x;
				xMax = startPoint.x;
				yMin = startPoint.y;
				yMax = startPoint.y;
				infolabel.showLabel(" 0,0   0,0°", 175);
				break;

			case MODE_ERASER : 
				erBreite    = erasers[EraseStyle].getWidth();
				erHoehe     = erasers[EraseStyle].getHeight();
				break;

			case MODE_MOVER : 
				dragged = true;
				break;

			case MODE_NOTES : 
				endPoint 	= e.getPoint();
				eineNote = Parent.MenuPanel.note.getValue();
				NotePics.NotenBilder[eineNote].setColor( image.getPenColor() );
				repaint();
				break;

			case MODE_MUSIC : 
				endPoint 	= e.getPoint();
				eineMusic   = Parent.MenuPanel.music.getValue();
				NotePics.MusicBilder[eineMusic].setColor( image.getPenColor() );
				repaint();
				break;

		}	// switch
		}   // if Button=1
	}

	public void mouseDragged(MouseEvent e)
	{	int ydiff;

		if ( mouseaction ) {

		switch(boardMode) {

			case MODE_PENCIL : 
				dragged 	= true;
				endPoint 	= e.getPoint();
				image.drawLine(startPoint.x, offset+startPoint.y, 
				                endPoint.x,   offset+endPoint.y       );
				repaint();
				startPoint 	= endPoint;
				break;

			case MODE_LINEAL : 
				dragged 	= true;
				endPoint = e.getPoint();
				xMin = Math.min(xMin, endPoint.x);
				xMax = Math.max(xMax, endPoint.x);
				yMin = Math.min(yMin, endPoint.y);
				yMax = Math.max(yMax, endPoint.y);
				infolabel.setText(getLineLength()+" "+getLineAngle());
				repaint();
				break;

			case MODE_ERASER : 
				dragged 	= true;
				endPoint = e.getPoint();
				image.clear( endPoint.x-erBreite/2,offset+endPoint.y-erHoehe/2,
				             erBreite,erHoehe );
				repaint(endPoint.x-erBreite/2,endPoint.y-erHoehe/2,erBreite,erHoehe);
				break;

			case MODE_MOVER : 
				dragged 	= true;
				System.out.println("start dragging");
				endPoint = e.getPoint();
				ydiff = startPoint.y-endPoint.y;
				if ( ydiff != 0 ) { scroll(ydiff); }
				startPoint 	= endPoint;
				System.out.println("end dragging");
				break;

			case MODE_NOTES : 
				endPoint 	= e.getPoint();
				repaint();
				break;

			case MODE_MUSIC : 
				endPoint 	= e.getPoint();
				repaint();
				break;

		}
		}
	}

	public void mouseReleased(MouseEvent e)
	{
		if ( e.getButton() == 1 ) {

		switch(boardMode) {

			case MODE_PENCIL : 
				if ( !dragged ) {
					image.setPointStroke();
					image.drawLine(endPoint.x, offset+endPoint.y, 
					                endPoint.x, offset+endPoint.y);
					repaint();
					image.setStroke();
				}; break;

			case MODE_LINEAL : 
				if ( dragged ) {
					image.drawLine(startPoint.x, offset+startPoint.y, 
					                endPoint.x, offset+endPoint.y);
				} else { 
					image.setPointStroke();
					image.drawLine(endPoint.x, offset+endPoint.y, 
					               endPoint.x, offset+endPoint.y);
					repaint();
					image.setStroke();
				}; 

				repaint();
				infolabel.hideLabel();
				break;

			case MODE_ERASER : 
				if ( !dragged ) {
				image.clear( endPoint.x-erBreite/2,offset+endPoint.y-erHoehe/2,
				             erBreite,erHoehe );
				repaint(endPoint.x-erBreite/2,endPoint.y-erHoehe/2,erBreite,erHoehe);
				}; break;

			case MODE_MOVER : 
				dragged = false;
				repaint();
				break;

			case MODE_NOTES : 
				image.drawImage(NotePics.NotenBilder[eineNote].getImage(), 
				                endPoint.x-NotePics.NotenBilder[eineNote].getOffsetX(), 
				                offset+endPoint.y-NotePics.NotenBilder[eineNote].getOffsetY() );
				endPoint 	= null;
				repaint();
				break;

			case MODE_MUSIC : 
				image.drawImage(NotePics.MusicBilder[eineMusic].getImage(), 
				                endPoint.x-NotePics.MusicBilder[eineMusic].getOffsetX(), 
				                offset+endPoint.y-NotePics.MusicBilder[eineMusic].getOffsetY() );
				endPoint 	= null;
				repaint();
				break;

		}

		startPoint = null; endPoint = null;
		mouseaction = false;

		}
	}


	public void mouseWheelMoved(MouseWheelEvent e) {
		scroll(wheelY*e.getWheelRotation());
	}



	public void paintComponent (Graphics g){

		super.paintComponent(g);

		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		g.drawImage(background.getSubimage(0, offset, getWidth(), getHeight()), 0, 0, null);

		g.drawImage(image.getSubimage(0, offset, getWidth(), getHeight()), 0, 0, null);


		if ( boardMode == MODE_LINEAL ) {
			if (startPoint != null && endPoint != null)
			{
				g2d.setColor( image.getPenColor() );
				g2d.setStroke( image.getStroke() );
				g2d.drawLine(startPoint.x, startPoint.y, endPoint.x, endPoint.y);
			}
			
		}

		if ( boardMode == MODE_NOTES ) {
			if (endPoint != null)
			{
				g2d.drawImage(NotePics.NotenBilder[eineNote].getImage(), 
				              endPoint.x-NotePics.NotenBilder[eineNote].getOffsetX(), 
				              endPoint.y-NotePics.NotenBilder[eineNote].getOffsetY(), null);
			}
		}

		if ( boardMode == MODE_MUSIC ) {
			if (endPoint != null)
			{
				g2d.drawImage(NotePics.MusicBilder[eineMusic].getImage(), 
				              endPoint.x-NotePics.MusicBilder[eineMusic].getOffsetX(), 
				              endPoint.y-NotePics.MusicBilder[eineMusic].getOffsetY(), null);
			}
		}

		if ( boardMode == MODE_GEODR ) {
			if ( Geodreieck.isRotating() ) {
			}
		}

		if ( boardMode == MODE_MOVER ) {

				g2d.setColor(cMoverBack);
				g2d.fillRect(moveOffset, getHeight()-moveOffset-moveHeight, moveWidth, moveHeight);
				g2d.setColor(Color.WHITE);
				g2d.fillRect(	moveOffset, 
								getHeight()-moveOffset-moveHeight+(offset*moveHeight)/maxY, 
								moveWidth, 
								(getHeight()*moveHeight)/maxY
							);

				g2d.setColor(Color.BLACK);
				g2d.drawRect(	moveOffset, 
								getHeight()-moveOffset-moveHeight+(offset*moveHeight)/maxY, 
								moveWidth-1, 
								(getHeight()*moveHeight)/maxY
							);
		}


	}
		
	public void actionPerformed (ActionEvent event) {
		// System.out.println("Selected: " + event.getActionCommand());	

		if (event.getSource() == popup.exitItem) {
			Parent.exit();
		}

		if (event.getSource() == popup.infoItem) {
			Parent.infoDialog.showInfo();
		}

	}
	
}
