import java.io.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.imageio.*;
import java.net.URL;
import java.util.*;

public class JInfoDialog extends JDialog {

	I18N messages = I18N.getInstance();

	JWB_Tafel parent;
	
	Image icon;

	//-------------------------------------------------------------

	public class JAboutPanel extends JPanel {

		public JAboutPanel() {
			setBackground(Color.WHITE);
			setOpaque(true);
			setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
			add(Box.createVerticalGlue());

			JLabel logo = new JLabel( new ImageIcon(icon), JLabel.CENTER );
			logo.setMaximumSize(new Dimension(9999,logo.getPreferredSize().height));
			add( logo );
			add(Box.createVerticalGlue());
			
			add( new JLabel(
				"<html><center><b><font size=+2>JWB-Tafel</font></b></center></html>",
				JLabel.CENTER) );
			add(Box.createVerticalGlue());

			add( new JLabel(
				"<html><center>"+messages.getStr("about.description")+"</center></html>",
				JLabel.CENTER) );
			add(Box.createVerticalGlue());

			add(new JLabel(
				"<html><center><b>Version 0.8 - 25.07.2015</b></center></html>",
				JLabel.CENTER) );
			add(Box.createVerticalGlue());

			add( new JLinkLabel(
				"www.kuhmann.de/jwb-tools/jwb-tafel/", JLabel.CENTER) );
			add(Box.createVerticalGlue());
			
		}
	}

	//-------------------------------------------------------------

	public class JLicensePane extends JEditorPane {

		public JLicensePane() {
			setEditable(false);
			java.net.URL helpURL = getClass().getResource("html/gpl-3.0-standalone.html");
			if (helpURL != null) {
				try {
					setPage(helpURL);
				} catch (IOException e) {
					System.err.println("Attempted to read a bad URL: " + helpURL);
				}
			} else {
				System.err.println("Couldn't find file: gpl-3.0-standalone.html");
			}
		}
	}

	//-------------------------------------------------------------

	public class JCreditsPane extends JEditorPane {

		public JCreditsPane() {
			setEditable(false);
			setOpaque(false);

			String creditsFile = "html/credits_"+Locale.getDefault().getLanguage()+".html";

			java.net.URL helpURL = getClass().getResource(creditsFile);

			if (helpURL != null) {
				try {
					setPage(helpURL);
				} catch (IOException e) {
					System.err.println("Attempted to read a bad URL: " + helpURL);
				}
			} else {

				creditsFile = "html/credits_en.html";
				helpURL = getClass().getResource(creditsFile);

				if (helpURL != null) {
					try {
						setPage(helpURL);
					} catch (IOException e) {
						System.err.println("Attempted to read a bad URL: " + helpURL);
					}
				} else {
					System.err.println("Couldn't find file: credits_en.html");
				}

			}
			
			addHyperlinkListener(new HyperlinkListener() {
                    @Override
                    public void hyperlinkUpdate(HyperlinkEvent hle) {
                        if (HyperlinkEvent.EventType.ACTIVATED.equals(hle.getEventType())) {
                            Desktop desktop = Desktop.getDesktop();
                            try {
                                desktop.browse(hle.getURL().toURI());
                            } catch (Exception ex) {
                                ex.printStackTrace();
                            }
                        }
                    }
                });
						
		}
	}

	//-------------------------------------------------------------

	public JInfoDialog(JWB_Tafel p) {
		setTitle( messages.getStr("about.about") ); 
		parent = p;
		try {
			icon = ImageIO.read(getClass().getResource("/pics/blackboard_64.png"));
			setIconImage(icon);
		} catch(IOException e) { e.printStackTrace();	}
				
		JTabbedPane tabbedPane = new JTabbedPane();
		
		tabbedPane.setBorder(null);
		tabbedPane.addTab(messages.getStr("about.about"), null, new JAboutPanel(), null);

		JScrollPane creditsScrollPane = new JScrollPane( new JCreditsPane() );

		tabbedPane.addTab(messages.getStr("about.credits"), null, creditsScrollPane, null);

		JScrollPane licenseScrollPane = new JScrollPane( new JLicensePane() );
		licenseScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		tabbedPane.addTab(messages.getStr("about.license"), null, licenseScrollPane, null);
		
        add("Center", tabbedPane );

		getContentPane().setPreferredSize(new Dimension(550, 400));
		getContentPane().setMinimumSize(new Dimension(550, 400));
		pack();
		
		setLocationRelativeTo(parent);
		setModal(true);
		setVisible(false);

	}

	public void showInfo() {
		setLocationRelativeTo(parent);
		setVisible(true);
	}

}

